/***************************************************************************
 *   Copyright (C) 2007 by Guy Rutenberg   *
 *   guyrutenberg@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>
#include <getopt.h>
#include "random.h"

using namespace std;
#define VERSION "1.0"

void GeneratePassword(int length, char *chars, int rand_type);
void PrintHelp();

int main(int argc, char **argv)
{
	int kind_password = 0;
	int length,num_of_passwords = 1;
	int security_level = 0;
	int optchr;
	char chars[96];
	Random random;
	enum {
		PASS_DEFAULT,
		PASS_ALPHANUMERIC,
		PASS_EXTENDEDSYMBOLS,
		PASS_UPPERCASE,
		PASS_LOWERCASE,
		PASS_NUMBERSONLY,
		PASS_HEX
	};
	const char* const short_options = "aCceHhfNn:V";
	const option long_options[] = {
			{"alphanumeric", no_argument, NULL,'a'},
			{"extended-symbols", no_argument, NULL, 'e'},
			{"uppercase",no_argument, &kind_password, PASS_UPPERCASE},
			{"lowercase",no_argument, &kind_password, PASS_LOWERCASE},
			{"numbers-only",no_argument, &kind_password, PASS_NUMBERSONLY},
			{"fast",no_argument, NULL, 'f'},
			{"number",required_argument,NULL,'n'},
			{"hexadecimal",no_argument, &kind_password, PASS_HEX},
			{"version", no_argument, NULL, 'V'},
			{"help", no_argument, NULL, 'h'},
			{NULL, 0, NULL, 0}
		};
	while((optchr = getopt_long(argc, argv, short_options, long_options, NULL)) != -1) {
		switch(optchr){
		case 'a':
			kind_password = PASS_ALPHANUMERIC;
			break;
		case 'e':
			kind_password = PASS_EXTENDEDSYMBOLS;
			break;
		case 'N':
			kind_password = PASS_NUMBERSONLY;
			break;
		case 'C':
			kind_password = PASS_UPPERCASE;
			break;
		case 'c':
			kind_password = PASS_LOWERCASE;
			break;
		case 'H':
			kind_password = PASS_HEX;
			break;
		case 'n':
			num_of_passwords = (atoi(optarg)<0) ? 1 : atoi(optarg);
			break;
		case 'f':
			security_level = 1;
			break;
		case 'V':
			cout<<"spassword " << VERSION << endl;
			cout<<"Copyright (c) 2006-2007 Guy Rutenberg"<<endl;
			cout<<"This is free software.  You may redistribute copies of it "
					"under the terms of\nthe GNU General Public License "
					"<http://www.gnu.org/licenses/gpl.html>. \nThere is NO "
					"WARRANTY, to the extent permitted by law."<<endl;
			return 0;
		case '?':
			cout << "See \'spassword --help\' for a list of valid options"<<endl;
			return 1;
		case 'h':
			PrintHelp();
			return 0;
		}		
	}
	if (optind<argc) {
		length = atoi(argv[optind]);
	} else {
		cout<<"You must specify password length"<<endl;
		return 1;
	}
	
	switch (kind_password) {
	case PASS_UPPERCASE:
		strcpy(chars,"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
		break;
	case PASS_LOWERCASE:
		strcpy(chars,"abcdefghijklmnopqrstuvwxyz0123456789");
		break;
	case PASS_NUMBERSONLY:
			strcpy(chars,"0123456789");
			break;
	case PASS_ALPHANUMERIC:
		strcpy(chars,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
				"0123456789");
		break;
	case PASS_EXTENDEDSYMBOLS:
		strcpy(chars,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
				"0123456789!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|} ~");
		break;
	case PASS_HEX:
		strcpy(chars,"0123456789ABCDEF");
		break;
	default:
		strcpy(chars,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
				"0123456789!@#$%^&*?_~");
	}
	for (int i = 0; i<num_of_passwords; i++) {
		GeneratePassword(length,chars, security_level);
	}
	
	return 0;
}

void PrintHelp()
{
	cout<<"usage: spassword [options] LENGTH"<<endl;
	cout<<"A secure (cryptography strong) password generator."<<endl<<endl;
	cout<<"options:"<<endl;
	cout<<"  -a, --alphanumeric   Generate passwords consisting of alphanumeric characters\n"
		"                       only"<<endl;
	cout<<"  -e, --extended-symbols\n"
		"                       Generate passwords consisting of alphanumeric characters\n"
		"                       and extended special symbol set:\n"
		"                       !\"#$%&'()*+,-./:;<=>?@[\\]^_`{|} ~"<<endl;
	cout<<"  -C, --uppercase      Generate passwords consisting of upper-case letters\n"
		"                       only"<<endl;
	cout<<"  -c, --lowercase      Generate passwords consisting of lower-case letters only"<<endl;
	cout<<"  -f, --fast           Use a faster but a bit less secure method "
		"to generate\n                       passwords"<<endl;
	cout<<" -H, --hexadecimal     Generate hexadecimal passwords"<<endl;
	cout<<"  -h, --help           Displays this help file and exits"<<endl;
	cout<<"  -N, --numbers-only   Generate passwords consisting only numbers"<<endl;
	cout<<"  -N NUM, --number=NUM\n"
		"                       Generate NUM passwords [default: 1]"<<endl;
	cout<<"  -V, --version        Displays version info and exits"<<endl;
	cout<<endl<<"Please report bugs to <guyrutenberg@gmai.com>."<<endl;
}
/**
 * Generate and print a new password.
 * \param length [int] the requested length of the password.
 * \param chars [char*] a pointer to a null-terminated string of valid
 * \param rand_type [int] determines what type of password generator will be used.
 *        If 0 Random::secure() will be used, if 1 Random::strong() will be used.
 * characters for the password.
 */
void GeneratePassword(int length, char *chars, int rand_type)
{
	register int i;
	unsigned int size = strlen(chars);
	unsigned int (Random::*prand)(unsigned int);
	Random random;
	if (rand_type) {
		prand = &Random::strong_range;
	} else {
		prand = &Random::secure_range;
	}
	
	for (i=0;i<length;i++) {
		cout<<chars[(random.*prand)(size)];
	}
	
	cout<<endl;
}
